/********************************************************************************
 * MIT Java MWE Library (jMWE) v1.0.1
 * Copyright (c) 2008-2011 Massachusetts Institute of Technology
 *
 * jMWE is distributed under the terms of the Creative Commons Attribution 3.0 
 * Unported License, which means it may be freely used for all purposes, as long 
 * as proper acknowledgment is made.  
 *******************************************************************************/

package edu.mit.jmwe.data;

import java.util.regex.Pattern;

/**
 * A MWE description ID that can be used to retrieve the MWE's lemma, part of
 * speech, and inflected form.
 * 
 * @author M.A. Finlayson
 * @version $Id: IMWEDescID.java 572 2011-05-05 19:40:35Z markaf $
 * @since jMWE 1.0.0
 */
public interface IMWEDescID extends IHasForm, IHasMWEPOS, Comparable<IMWEDescID> {

	/**
	 * Captures the format of a well-formed form associated with this ID. The
	 * form must have its parts delimited by underscores.
	 * 
	 * @since jMWE 1.0.0
	 */
	public Pattern formPattern = Pattern.compile("([\\S&&[^_]]+_)+([\\S&&[^_]]+)");

	/**
	 * Returns the root ID of this decription ID. The root ID contains the MWE's
	 * lemma and part of speech.
	 * 
	 * @return the root ID of this decription ID
	 * @since jMWE 1.0.0
	 */
	public IMWEDescID getRootID();

	/**
	 * Returns an inflected form of the MWE associated with this ID.
	 * 
	 * @return the inflected form of this MWE.
	 * @since jMWE 1.0.0
	 */
	public String getInflectedForm();
}
