/********************************************************************************
 * MIT Java MWE Library (jMWE) v1.0.1
 * Copyright (c) 2008-2011 Massachusetts Institute of Technology
 *
 * jMWE is distributed under the terms of the Creative Commons Attribution 3.0 
 * Unported License, which means it may be freely used for all purposes, as long 
 * as proper acknowledgment is made.  
 *******************************************************************************/

package edu.mit.jmwe.detect;

import edu.mit.jmwe.data.IMWE;
import edu.mit.jmwe.data.IToken;

/**
 * Represents a pattern according to which the parts of a multi-word expression
 * may be inflected.
 * 
 * @author N. Kulkarni
 * @version $Id: IInflectionRule.java 620 2011-05-08 21:13:58Z markaf $
 * @since jMWE 1.0.0
 */
public interface IInflectionRule {

	/**
	 * Returns <code>true</code> if this MWE follows the rule;
	 * <code>false</code> otherwise.
	 * 
	 * @param <T>
	 *            the type of tokens in the MWE
	 * @param mwe
	 *            the MWE to which the rule may/may not be applied
	 * @throws IllegalArgumentException
	 *             if this rule may not be applied to the given MWE. May only
	 *             throw this exception if the <code>matches(IMWE<T> mwe)</code>
	 *             method returns false.
	 * @return <code>true</code> if this MWE follows the rule;
	 *         <code>false</code> otherwise.
	 * @since jMWE 1.0.0
	 */
	public <T extends IToken> boolean isValid(IMWE<T> mwe);

	/**
	 * Returns <code>true</code> if the given MWE has the same syntax as this
	 * rule. In other words, returns <code>true</code> if this rule may be
	 * applied to the given MWE; <code>false</code> otherwise.
	 * 
	 * @param <T>
	 *            the type of tokens in the MWE
	 * @param mwe
	 *            the MWE to which the rule may/may not be applied
	 * @return returns <code>true</code> if this rule may be applied to the
	 *         given MWE, <code>false</code> otherwise.
	 * @since jMWE 1.0.0
	 */
	public <T extends IToken>  boolean matches(IMWE<T> mwe);

}
