/********************************************************************************
 * MIT Java MWE Library (jMWE) v1.0.1
 * Copyright (c) 2008-2011 Massachusetts Institute of Technology
 *
 * jMWE is distributed under the terms of the Creative Commons Attribution 3.0 
 * Unported License, which means it may be freely used for all purposes, as long 
 * as proper acknowledgment is made.  
 *******************************************************************************/

package edu.mit.jmwe.detect.score;

import edu.mit.jmwe.data.IMWE;
import edu.mit.jmwe.data.IMWEDesc;
import edu.mit.jmwe.data.IToken;

/**
 * A scorer that scores with the fraction of times it appears marked as an MWE,
 * as opposed to a run of unmarked tokens.
 * 
 * @param <T>
 *            the type of token used by this scorer
 * @author M.A. Finlayson
 * @version $Id: FractionAsMWEScore.java 608 2011-05-06 19:32:34Z markaf $
 * @since jMWE 1.0.0
 */
public class FractionAsMWEScore<T extends IToken> extends AbstractScorer<IMWE<T>> {
	
	//the singleton instance
	private static FractionAsMWEScore<?> instance = null;
	
	/**
	 * Returns the singleton instance of this class, instantiating if necessary.
	 *
	 * @return the singleton instance of this class
	 * @since jMWE 1.0.0
	 */
	@SuppressWarnings("unchecked")
	public static <T extends IToken> FractionAsMWEScore<T> getInstance() {
		if (instance == null)
			instance = new FractionAsMWEScore<IToken>();
		return (FractionAsMWEScore<T>)instance;
	}

	/**
	 * This constructor is marked protected so that this class may be
	 * subclassed, but not directly instantiated.
	 *
	 * @since jMWE 1.0.0
	 */
	protected FractionAsMWEScore() {}

	/* 
	 * (non-Javadoc)
	 *
	 * @see edu.mit.jmwe.detect.score.IScorer#score(java.lang.Object)
	 */
	public double score(IMWE<T> mwe) {
		IMWEDesc desc = mwe.getEntry();
		double numer = desc.getMarkedContinuous();
		double denom = numer + desc.getUnmarkedExact();
		if(Double.compare(denom, 0.0) == 0)
			return 1.0;
		return numer / denom;
	}

}
